import os, sys
import unittest
from sets import Set
import traceback

if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Testing                 import ZopeTestCase
from Products.CMFCore.utils  import getToolByName
from Products.CMFPlone.tests import PloneTestCase
from Products.Archetypes.tests.ArchetypesTestCase import ArcheSiteTestCase
from DateTime import DateTime

from Products.Archetypes.Extensions.Install import install as installAT
from Products.PloneStickies.Extensions.Install import install as installStickies

from Products.PloneStickies.config import *

ZopeTestCase.installProduct('PloneStickies')

# util for making content in a container
def makeContent(container, id, portal_type, title=None):
    container.invokeFactory(id=id, type_name=portal_type)
    o = getattr(container, id)
    if title is not None:
        o.setTitle(title)
    return o

class PloneStickiesToolTest(ArcheSiteTestCase):
    def afterSetUp(self):
        """
        Create some at content, and annotate it
        """
        ArcheSiteTestCase.afterSetUp(self)

        # use at demo types for testing,
        # since we need at dervied types for stickies to work
        # reinstall so we get the example types.  test off DDocument
        installAT(self.portal, include_demo = 1)
        # import pdb; pdb.set_trace()
        installStickies(self.portal)

        self._refreshSkinData()

        # stock our test site with a case, a question, and some assets
        self.loginPortalOwner()   # loging as owner wo we can create stuff

        doc1 = makeContent(self.portal, 'doc1', 'DDocument')

        assert doc1


    def test_toolInstalled(self):
        s_tool  =  getToolByName(self.portal, STICKIES_TOOL)
        assert s_tool
        
    def test_stickyCreation(self):
        """
        annotate a ddoc and verify that the relationship is intact
        """
        portal = self.portal
        doc1 = portal.doc1
        
        s_tool = getToolByName(portal, STICKIES_TOOL)
        # create new sticky note
        sticky  = s_tool.createNewSticky(doc1)

        # assert that sticky is connected to doc1
        refs = sticky.getRefs(relationship = STICKY_RELATIONSHIP)        
        assert refs[0] == doc1

        # assert that doc1 is connected to sticky
        backrefs = doc1.getBRefs(relationship = STICKY_RELATIONSHIP)
        assert backrefs[0] == sticky
        
    def test_stickyCreationByUID(self):
        """
        annotate a ddoc and verify that the relationship is intact
        """
        portal = self.portal
        doc1 = portal.doc1
        doc1UID = doc1.UID()
        
        s_tool = getToolByName(portal, STICKIES_TOOL)
        sticky  = s_tool.createNewSticky(doc1UID)
        refs = sticky.getRefs(relationship = STICKY_RELATIONSHIP)
        assert refs[0] == portal.doc1
        
    def test_stickyHiddenCreation(self):
        """
        annotate a ddoc and verify that the relationship is intact
        """
        portal = self.portal
        doc1 = portal.doc1
        
        s_tool = getToolByName(portal, STICKIES_TOOL)
        # create new sticky note
        hidden_sticky  = s_tool.createNewSticky(doc1)
        id = hidden_sticky.id

        assert id.startswith(".")

        open_sticky  = s_tool.createNewSticky(doc1, hidden=False)
        id = open_sticky.id
        assert not id.startswith(".")

        
    def test_deletSticky(self):
        portal = self.portal
        doc1 = portal.doc1
        doc1UID = doc1.UID()
        
        s_tool = getToolByName(portal, STICKIES_TOOL)
        sticky  = s_tool.createNewSticky(doc1UID)

        # assert that doc1 is connected to sticky
        backrefs = doc1.getBRefs(relationship = STICKY_RELATIONSHIP)
        assert backrefs[0] == sticky

        s_tool.deleteSticky(sticky.UID())

        # after deletion the sticky should be gone
        backrefs = doc1.getBRefs(relationship = STICKY_RELATIONSHIP)
        assert backrefs == []

    def test_findByTarget(self):
        portal = self.portal
        doc1 = portal.doc1
        
        s_tool = getToolByName(portal, STICKIES_TOOL)
        # create new sticky note
        sticky  = s_tool.createNewSticky(doc1)

        # do more testing here soon
        assert s_tool.findStickiesByTarget(doc1.UID())

    def test_findByMember(self):
        portal = self.portal
        doc1 = portal.doc1
        
        s_tool = getToolByName(portal, STICKIES_TOOL)
        # create new sticky note
        sticky  = s_tool.createNewSticky(doc1)

        # do more testing here soon
        assert s_tool.findStickiesByMember('portal_owner')

    def test_getSingleSticky(self):
        portal = self.portal
        doc1 = portal.doc1
        
        s_tool = getToolByName(portal, STICKIES_TOOL)
        # create new sticky note
        sticky  = s_tool.createNewSticky(doc1)

        foundExplicitMemberId = s_tool.getSticky(doc1.UID(), 'portal_owner')

        assert foundExplicitMemberId == sticky

        foundImplicitMemberId = s_tool.getSticky(doc1.UID())

        assert foundImplicitMemberId == sticky

    def test_getTarget(self):
        portal = self.portal
        doc1 = portal.doc1

        s_tool = getToolByName(portal, STICKIES_TOOL)
        # create new sticky note
        sticky  = s_tool.createNewSticky(doc1)

        target = sticky.getTarget()
        assert target == doc1

        
        
def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(PloneStickiesToolTest))
    return suite

if __name__ == '__main__':
    framework()
